#
# Copyright (c) 2022 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#

#
# Find the ncursesw includes and library.
#
# This module defines:
#   NCURSESW_INCLUDE_DIR
#   NCURSESW_LIBRARY
#   NCURSESW_FOUND
#
# Use the following variables to help locate the library and header files.
#   WITH_NCURSESW_LIB=x      - directory containing libncurses*.so
#   WITH_NCURSESW_INCLUDES=x - directory containing ncurses*.h
#   WITH_NCURSESW=x          - same as setting WITH_NCURSESW_LIB=x/lib and
#                              WITH_NCURSESW_INCLUDES=x/include
#

if (WITH_NCURSESW)
	if (NOT WITH_NCURSESW_LIB)
		set(WITH_NCURSESW_LIB "${WITH_NCURSESW}/lib")
	endif()
	if (NOT WITH_NCURSESW_INCLUDES)
		set(WITH_NCURSESW_INCLUDES "${WITH_NCURSESW}/include")
	endif()
endif()

# find the correct library (ncursesw, ncurses)
find_library(NCURSESW_LIBRARY
	NAMES ncursesw ncurses
	PATHS "${WITH_NCURSES_LIB}"
	REQUIRED
)

# find the correct header (ncursesw.h, ncurses.h)
find_path(NCURSESW_INCLUDE_DIR
	NAMES
		ncursesw/ncurses.h ncursesw/curses.h ncursesw.h cursesw.h
		ncurses/ncurses.h ncurses/curses.h ncurses.h curses.h
	PATHS "${WITH_NCURSESW_INCLUDES}"
	REQUIRED
)

# TODO: check that the found lib & header is usable

include(FindPackageHandleStandardArgs)
find_package_handle_standard_args(ncursesw DEFAULT_MSG NCURSESW_LIBRARY
	NCURSESW_INCLUDE_DIR)
