/*
 * Copyright (c) 2020-2021 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef __HLQ_H
#define __HLQ_H

#include <jeffpc/buffer.h>
#include <jeffpc/uuid.h>
#include <jeffpc/val.h>

#include <hlog/qso-serde.h>

extern int hlq_append_header(struct buffer *buf, const struct xuuid *uuid);

extern int hlq_append_kv_null(struct buffer *buf, enum qso_ns ns,
			      const char *key);
extern int hlq_append_kv_int(struct buffer *buf, enum qso_ns ns,
			     const char *key, uint64_t value);
extern int hlq_append_kv_cstr(struct buffer *buf, enum qso_ns ns,
			      const char *key, const char *value);
extern int hlq_append_kv_time(struct buffer *buf, enum qso_ns ns,
			      const char *key, const struct time *time);
extern int hlq_append_kv_blob(struct buffer *buf, enum qso_ns ns,
			      const char *key, const void *value, size_t length);
extern int hlq_append_kv(struct buffer *buf, enum qso_ns ns, const char *key,
			 struct val *val);

#endif
