/*
 * Copyright (c) 2020-2021 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef __ADIF_H
#define __ADIF_H

#include <jeffpc/buffer.h>
#include <jeffpc/val.h>

#include <hlog/qso.h>

extern int adif_append_header(struct buffer *buf);
extern int adif_append_qso(struct buffer *buf, const struct qso *qso,
			   bool with_header, bool strict_mode);
extern int adif_append_kv(struct buffer *buf, const char *key, struct val *val);
extern int adif_append_kv_int(struct buffer *buf, const char *key,
			      uint64_t value);
extern int adif_append_kv_cstr(struct buffer *buf, const char *key,
			       const char *value);
extern int adif_append_kv_time(struct buffer *buf, const char *key,
			       const struct time *time);
extern int adif_append_kv_date(struct buffer *buf, const char *key,
			       const struct time *time);
extern int adif_append_kv_blob(struct buffer *buf, const char *key,
			       const void *value, size_t length);

extern struct buffer *adif_write_qso(const struct qso *qso);

/*
 * ADIF mode/submode handling
 */
/* returns 0 on success, negated errno otherwise */
extern int adif_mode_lookup_by_hlog_mode(const char *hlog_mode,
					 const char **adif_mode_r,
					 const char **adif_submode_r);
/* returns string on success, negated errno otherwise */
extern const char *adif_mode_lookup_by_adif_mode(const char *adif_mode,
						 const char *adif_submode);
/* returns val on success, negated errno otherwise */
extern struct str *adif_mode_lookup_mode_by_hlog_mode(const char *hlog_mode);
/* returns val or NULL on success, negated errno otherwise */
extern struct str *adif_mode_lookup_submode_by_hlog_mode(const char *hlog_mode);

#endif
