/*
 * Copyright (c) 2020-2022 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef __QSO_SERDE_H
#define __QSO_SERDE_H

#include <hlog/qso.h>

enum qso_ns {
	QSO_NS_DEBUG,
	QSO_NS_QSO,
	QSO_NS_TX,
	QSO_NS_RX,
	QSO_NS_ADDL,
};

/* pack & unpack whole struct qso */
extern struct buffer *qso_pack_hlq(const struct qso *qso);
extern int qso_unpack_hlq(struct qso *qso, const void *buf, size_t len);

/* partial parsing helpers */
extern int hlq_parse_header(struct buffer *buf, struct xuuid *uuid_r);
extern struct val *hlq_parse_kv(struct buffer *buf, int ver, enum qso_ns *ns_r,
				const char **key_start_r, const char **key_end);
extern int qso_set_hlq_item(struct qso *qso, enum qso_ns ns,
			    const char *key_start, const char *key_end,
			    struct val *val);

#endif
