/*
 * Copyright (c) 2021 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <jeffpc/types.h>

STATIC_ASSERT(LCD_CHAR_WIDTH == 5);

#define b_____	0x00
#define b____X	0x01
#define b___X_	0x02
#define b___XX	0x03
#define b__X__	0x04
#define b__X_X	0x05
#define b__XX_	0x06
#define b__XXX	0x07
#define b_X___	0x08
#define b_X__X	0x09
#define b_X_X_	0x0a
#define b_X_XX	0x0b
#define b_XX__	0x0c
#define b_XX_X	0x0d
#define b_XXX_	0x0e
#define b_XXXX	0x0f
#define bX____	0x10
#define bX___X	0x11
#define bX__X_	0x12
#define bX__XX	0x13
#define bX_X__	0x14
#define bX_X_X	0x15
#define bX_XX_	0x16
#define bX_XXX	0x17
#define bXX___	0x18
#define bXX__X	0x19
#define bXX_X_	0x1a
#define bXX_XX	0x1b
#define bXXX__	0x1c
#define bXXX_X	0x1d
#define bXXXX_	0x1e
#define bXXXXX	0x1f

const uint8_t lcd_font[256][LCD_CHAR_HEIGHT] = {
	[0x10] = { /* vline left X */
		bX____,
		bX____,
		bX____,
		bX____,
		bX____,
		bX____,
		bX____,
	},

	[0x11] = { /* vline left 2 */
		bXX___,
		bXX___,
		bXX___,
		bXX___,
		bXX___,
		bXX___,
		bXX___,
	},

	[0x12] = { /* vline left 3 */
		bXXX__,
		bXXX__,
		bXXX__,
		bXXX__,
		bXXX__,
		bXXX__,
		bXXX__,
	},

	[0x13] = { /* vline left 4 */
		bXXXX_,
		bXXXX_,
		bXXXX_,
		bXXXX_,
		bXXXX_,
		bXXXX_,
		bXXXX_,
	},

	[0x14] = { /* full block */
		bXXXXX,
		bXXXXX,
		bXXXXX,
		bXXXXX,
		bXXXXX,
		bXXXXX,
		bXXXXX,
	},

	[0x15] = { /* vline right 4 */
		b_XXXX,
		b_XXXX,
		b_XXXX,
		b_XXXX,
		b_XXXX,
		b_XXXX,
		b_XXXX,
	},

	[0x16] = { /* vline right 3 */
		b__XXX,
		b__XXX,
		b__XXX,
		b__XXX,
		b__XXX,
		b__XXX,
		b__XXX,
	},

	[0x17] = { /* vline right 2 */
		b___XX,
		b___XX,
		b___XX,
		b___XX,
		b___XX,
		b___XX,
		b___XX,
	},

	[0x18] = { /* vline right X */
		b____X,
		b____X,
		b____X,
		b____X,
		b____X,
		b____X,
		b____X,
	},

	[0x19] = { /* note */
		b__X__,
		b__XX_,
		b__X_X,
		b__X_X,
		b_XX_X,
		bXXX__,
		bXX___,
	},

	[0x1a] = { /* degrees C */
		bXX___,
		bXX___,
		b_____,
		b__XXX,
		b_X___,
		b_X___,
		b__XXX,
	},

	[0x1b] = { /* degrees F */
		bXX___,
		bXX___,
		b_____,
		b_XXXX,
		b_X___,
		b_XXX_,
		b_X___,
	},

	[0x1c] = { /* down arrow */
		b_____,
		b_____,
		bXXXXX,
		b_XXX_,
		b__X__,
		b_____,
		b_____,
	},

	[0x1d] = { /* right arrow */
		b_____,
		b_X___,
		b_XX__,
		b_XXX_,
		b_XX__,
		b_X___,
		b_____,
	},

	[0x1e] = { /* left arrow */
		b_____,
		b___X_,
		b__XX_,
		b_XXX_,
		b__XX_,
		b___X_,
		b_____,
	},

	[0x1f] = { /* up arrow */
		b_____,
		b_____,
		b__X__,
		b_XXX_,
		bXXXXX,
		b_____,
		b_____,
	},

	[0x20] = { /* space */
		b_____,
		b_____,
		b_____,
		b_____,
		b_____,
		b_____,
		b_____,
	},

	[0x21] = { /* ! */
		b__X__,
		b__X__,
		b__X__,
		b__X__,
		b_____,
		b_____,
		b__X__,
	},

	[0x22] = { /* _ */
		b_X_X_,
		b_X_X_,
		b_X_X_,
		b_____,
		b_____,
		b_____,
		b_____,
	},

	[0x23] = { /* X */
		b_X_X_,
		b_X_X_,
		bXXXXX,
		b_X_X_,
		bXXXXX,
		b_X_X_,
		b_X_X_,
	},

	[0x24] = { /* $ */
		b__X__,
		b_XXXX,
		bX_X__,
		b_XXX_,
		b__X_X,
		bXXXX_,
		b__X__,
	},

	[0x25] = { /* % */
		bXX___,
		bXX__X,
		b___X_,
		b__X__,
		b_X___,
		bX__XX,
		b___XX,
	},

	[0x26] = { /* & */
		b_XX__,
		bX__X_,
		bX_X__,
		b_X___,
		bX_X_X,
		bX__X_,
		b_XX_X,
	},

	[0x27] = { /* ' */
		b_XX__,
		b__X__,
		b_X___,
		b_____,
		b_____,
		b_____,
		b_____,
	},

	[0x28] = { /* ( */
		b___X_,
		b__X__,
		b_X___,
		b_X___,
		b_X___,
		b__X__,
		b___X_,
	},

	[0x29] = { /* ) */
		b_X___,
		b__X__,
		b___X_,
		b___X_,
		b___X_,
		b__X__,
		b_X___,
	},

	[0x2a] = { /* * */
		b_____,
		b__X__,
		bX_X_X,
		b_XXX_,
		bX_X_X,
		b__X__,
		b_____,
	},

	[0x2b] = { /* + */
		b_____,
		b__X__,
		b__X__,
		bXXXXX,
		b__X__,
		b__X__,
		b_____,
	},

	[0x2c] = { /* , */
		b_____,
		b_____,
		b_____,
		b_____,
		b_XX__,
		b__X__,
		b_X___,
	},

	[0x2d] = { /* - */
		b_____,
		b_____,
		b_____,
		bXXXXX,
		b_____,
		b_____,
		b_____,
	},

	[0x2e] = { /* . */
		b_____,
		b_____,
		b_____,
		b_____,
		b_____,
		b_XX__,
		b_XX__,
	},

	[0x2f] = { /* / */
		b_____,
		b____X,
		b___X_,
		b__X__,
		b_X___,
		bX____,
		b_____,
	},

	[0x30] = { /* _ */
		b_XXX_,
		bX___X,
		bX__XX,
		bX_X_X,
		bXX__X,
		bX___X,
		b_XXX_,
	},

	[0x31] = { /* X */
		b__X__,
		b_XX__,
		b__X__,
		b__X__,
		b__X__,
		b__X__,
		b_XXX_,
	},

	[0x32] = { /* 2 */
		b_XXX_,
		bX___X,
		b____X,
		b___X_,
		b__X__,
		b_X___,
		bXXXXX,
	},

	[0x33] = { /* 3 */
		bXXXXX,
		b___X_,
		b__X__,
		b___X_,
		b____X,
		bX___X,
		b_XXX_,
	},

	[0x34] = { /* 4 */
		b___X_,
		b__XX_,
		b_X_X_,
		bX__X_,
		bXXXXX,
		b___X_,
		b___X_,
	},

	[0x35] = { /* 5 */
		bXXXXX,
		bX____,
		bXXXX_,
		b____X,
		b____X,
		bX___X,
		b_XXX_,
	},

	[0x36] = { /* 6 */
		b__XX_,
		b_X___,
		bX____,
		bXXXX_,
		bX___X,
		bX___X,
		b_XXX_,
	},

	[0x37] = { /* 7 */
		bXXXXX,
		b____X,
		b___X_,
		b__X__,
		b_X___,
		b_X___,
		b_X___,
	},

	[0x38] = { /* 8 */
		b_XXX_,
		bX___X,
		bX___X,
		b_XXX_,
		bX___X,
		bX___X,
		b_XXX_,
	},

	[0x39] = { /* 9 */
		b_XXX_,
		bX___X,
		bX___X,
		b_XXXX,
		b____X,
		b___X_,
		b_XX__,
	},

	[0x3a] = { /* : */
		b_____,
		b_XX__,
		b_XX__,
		b_____,
		b_XX__,
		b_XX__,
		b_____,
	},

	[0x3b] = { /* ; */
		b_____,
		b_XX__,
		b_XX__,
		b_____,
		b_XX__,
		b__X__,
		b_X___,
	},

	[0x3c] = { /* < */
		b___X_,
		b__X__,
		b_X___,
		bX____,
		b_X___,
		b__X__,
		b___X_,
	},

	[0x3d] = { /* = */
		b_____,
		b_____,
		bXXXXX,
		b_____,
		bXXXXX,
		b_____,
		b_____,
	},

	[0x3e] = { /* > */
		b_X___,
		b__X__,
		b___X_,
		b____X,
		b___X_,
		b__X__,
		b_X___,
	},

	[0x3f] = { /* ? */
		b_XXX_,
		bX___X,
		b____X,
		b___X_,
		b__X__,
		b_____,
		b__X__,
	},

	[0x40] = { /* @ */
		b_XXX_,
		bX___X,
		b____X,
		b_XX_X,
		bX_X_X,
		bX_X_X,
		b_XXX_,
	},

	[0x41] = { /* A */
		b_XXX_,
		bX___X,
		bX___X,
		bX___X,
		bXXXXX,
		bX___X,
		bX___X,
	},

	[0x42] = { /* B */
		bXXXX_,
		bX___X,
		bX___X,
		bXXXX_,
		bX___X,
		bX___X,
		bXXXX_,
	},

	[0x43] = { /* C */
		b_XXX_,
		bX___X,
		bX____,
		bX____,
		bX____,
		bX___X,
		b_XXX_,
	},

	[0x44] = { /* D */
		bXXX__,
		bX__X_,
		bX___X,
		bX___X,
		bX___X,
		bX__X_,
		bXXX__,
	},

	[0x45] = { /* E */
		bXXXXX,
		bX____,
		bX____,
		bXXXX_,
		bX____,
		bX____,
		bXXXXX,
	},

	[0x46] = { /* F */
		bXXXXX,
		bX____,
		bX____,
		bXXXX_,
		bX____,
		bX____,
		bX____,
	},

	[0x47] = { /* G */
		b_XXX_,
		bX___X,
		bX____,
		bX_XXX,
		bX___X,
		bX___X,
		b_XXXX,
	},

	[0x48] = { /* H */
		bX___X,
		bX___X,
		bX___X,
		bXXXXX,
		bX___X,
		bX___X,
		bX___X,
	},

	[0x49] = { /* I */
		b_XXX_,
		b__X__,
		b__X__,
		b__X__,
		b__X__,
		b__X__,
		b_XXX_,
	},

	[0x4a] = { /* J */
		b__XXX,
		b___X_,
		b___X_,
		b___X_,
		b___X_,
		bX__X_,
		b_XX__,
	},

	[0x4b] = { /* K */
		bX___X,
		bX__X_,
		bX_X__,
		bXX___,
		bX_X__,
		bX__X_,
		bX___X,
	},

	[0x4c] = { /* L */
		bX____,
		bX____,
		bX____,
		bX____,
		bX____,
		bX____,
		bXXXXX,
	},

	[0x4d] = { /* M */
		bX___X,
		bXX_XX,
		bX_X_X,
		bX_X_X,
		bX___X,
		bX___X,
		bX___X,
	},

	[0x4e] = { /* N */
		bX___X,
		bX___X,
		bXX__X,
		bX_X_X,
		bX__XX,
		bX___X,
		bX___X,
	},

	[0x4f] = { /* O */
		b_XXX_,
		bX___X,
		bX___X,
		bX___X,
		bX___X,
		bX___X,
		b_XXX_,
	},

	[0x50] = { /* P */
		bXXXX_,
		bX___X,
		bX___X,
		bXXXX_,
		bX____,
		bX____,
		bX____,
	},

	[0x51] = { /* Q */
		b_XXX_,
		bX___X,
		bX___X,
		bX___X,
		bX_X_X,
		bX__X_,
		b_XX_X,
	},

	[0x52] = { /* R */
		bXXXX_,
		bX___X,
		bX___X,
		bXXXX_,
		bX_X__,
		bX__X_,
		bX___X,
	},

	[0x53] = { /* S */
		b_XXXX,
		bX____,
		bX____,
		b_XXX_,
		b____X,
		b____X,
		bXXXX_,
	},

	[0x54] = { /* T */
		bXXXXX,
		b__X__,
		b__X__,
		b__X__,
		b__X__,
		b__X__,
		b__X__,
	},

	[0x55] = { /* U */
		bX___X,
		bX___X,
		bX___X,
		bX___X,
		bX___X,
		bX___X,
		b_XXX_,
	},

	[0x56] = { /* V */
		bX___X,
		bX___X,
		bX___X,
		bX___X,
		bX___X,
		b_X_X_,
		b__X__,
	},

	[0x57] = { /* W */
		bX___X,
		bX___X,
		bX___X,
		bX_X_X,
		bX_X_X,
		bX_X_X,
		b_X_X_,
	},

	[0x58] = { /* X */
		bX___X,
		bX___X,
		b_X_X_,
		b__X__,
		b_X_X_,
		bX___X,
		bX___X,
	},

	[0x59] = { /* Y */
		bX___X,
		bX___X,
		bX___X,
		b_X_X_,
		b__X__,
		b__X__,
		b__X__,
	},

	[0x5a] = { /* Z */
		bXXXXX,
		b____X,
		b___X_,
		b__X__,
		b_X___,
		bX____,
		bXXXXX,
	},

	[0x5b] = { /* [ */
		b_XXX_,
		b_X___,
		b_X___,
		b_X___,
		b_X___,
		b_X___,
		b_XXX_,
	},

	[0x5c] = { /* Yen */
		bX___X,
		b_X_X_,
		bXXXXX,
		b__X__,
		bXXXXX,
		b__X__,
		b__X__,
	},

	[0x5d] = { /* ] */
		b_XXX_,
		b___X_,
		b___X_,
		b___X_,
		b___X_,
		b___X_,
		b_XXX_,
	},

	[0x5e] = { /* ^ */
		b__X__,
		b_X_X_,
		bX___X,
		b_____,
		b_____,
		b_____,
		b_____,
	},

	[0x5f] = { /* _ */
		b_____,
		b_____,
		b_____,
		b_____,
		b_____,
		b_____,
		bXXXXX,
	},

	[0x60] = { /* ` */
		b_X___,
		b__X__,
		b___X_,
		b_____,
		b_____,
		b_____,
		b_____,
	},

	[0x61] = { /* a */
		b_____,
		b_____,
		b_XXX_,
		b____X,
		b_XXXX,
		bX___X,
		b_XXXX,
	},

	[0x62] = { /* b */
		bX____,
		bX____,
		bX____,
		bX_XX_,
		bXX__X,
		bX___X,
		bXXXX_,
	},

	[0x63] = { /* c */
		b_____,
		b_____,
		b_XXX_,
		bX____,
		bX____,
		bX___X,
		b_XXX_,
	},

	[0x64] = { /* d */
		b____X,
		b____X,
		b____X,
		b_XX_X,
		bX__XX,
		bX___X,
		b_XXXX,
	},

	[0x65] = { /* e */
		b_____,
		b_____,
		b_XXX_,
		bX___X,
		bXXXXX,
		bX____,
		b_XXX_,
	},

	[0x66] = { /* f */
		b__XX_,
		b_X__X,
		b_X___,
		bXXX__,
		b_X___,
		b_X___,
		b_X___,
	},

	[0x67] = { /* g */
		b_____,
		b_XXXX,
		bX___X,
		bX___X,
		b_XXXX,
		b____X,
		b_XXX_,
	},

	[0x68] = { /* h */
		bX____,
		bX____,
		bX_XX_,
		bXX__X,
		bX___X,
		bX___X,
		bX___X,
	},

	[0x69] = { /* i */
		b_____,
		b__X__,
		b_____,
		b__X__,
		b__X__,
		b__X__,
		b__X__,
	},

	[0x6a] = { /* j */
		b___X_,
		b_____,
		b__XX_,
		b___X_,
		b___X_,
		bX__X_,
		b_XX__,
	},

	[0x6b] = { /* k */
		bX____,
		bX____,
		bX__X_,
		bX_X__,
		bXX___,
		bX_X__,
		bX__X_,
	},

	[0x6c] = { /* l */
		b_XX__,
		b__X__,
		b__X__,
		b__X__,
		b__X__,
		b__X__,
		b_XXX_,
	},

	[0x6d] = { /* m */
		b_____,
		b_____,
		bXX_X_,
		bX_X_X,
		bX_X_X,
		bX___X,
		bX___X,
	},

	[0x6e] = { /* n */
		b_____,
		b_____,
		bX_XX_,
		bXX__X,
		bX___X,
		bX___X,
		bX___X,
	},

	[0x6f] = { /* o */
		b_____,
		b_____,
		b_XXX_,
		bX___X,
		bX___X,
		bX___X,
		b_XXX_,
	},

	[0x70] = { /* p */
		b_____,
		b_____,
		bXXXX_,
		bX___X,
		bXXXX_,
		bX____,
		bX____,
	},

	[0x71] = { /* q */
		b_____,
		b_____,
		b_XX_X,
		bX__XX,
		b_XXXX,
		b____X,
		b____X,
	},

	[0x72] = { /* r */
		b_____,
		b_____,
		bX_XX_,
		bXX__X,
		bX____,
		bX____,
		bX____,
	},

	[0x73] = { /* s */
		b_____,
		b_____,
		b_XXX_,
		bX____,
		b_XXX_,
		b____X,
		bXXXX_,
	},

	[0x74] = { /* t */
		b_X___,
		b_X___,
		bXXX__,
		b_X___,
		b_X___,
		b_X__X,
		b__XX_,
	},

	[0x75] = { /* u */
		b_____,
		b_____,
		bX___X,
		bX___X,
		bX___X,
		bX__XX,
		b_XX_X,
	},

	[0x76] = { /* v */
		b_____,
		b_____,
		bX___X,
		bX___X,
		bX___X,
		b_X_X_,
		b__X__,
	},

	[0x77] = { /* w */
		b_____,
		b_____,
		bX___X,
		bX___X,
		bX_X_X,
		bX_X_X,
		b_X_X_,
	},

	[0x78] = { /* x */
		b_____,
		b_____,
		bX___X,
		b_X_X_,
		b__X__,
		b_X_X_,
		bX___X,
	},

	[0x79] = { /* y */
		b_____,
		b_____,
		bX___X,
		bX___X,
		b_XXXX,
		b____X,
		b_XXX_,
	},

	[0x7a] = { /* z */
		b_____,
		b_____,
		bXXXXX,
		b___X_,
		b__X__,
		b_X___,
		bXXXXX,
	},

	[0x7b] = { /* { */
		b___X_,
		b__X__,
		b__X__,
		b_X___,
		b__X__,
		b__X__,
		b___X_,
	},

	[0x7c] = { /* | */
		b__X__,
		b__X__,
		b__X__,
		b__X__,
		b__X__,
		b__X__,
		b__X__,
	},

	[0x7d] = { /* } */
		b_X___,
		b__X__,
		b__X__,
		b___X_,
		b__X__,
		b__X__,
		b_X___,
	},

	[0x7e] = { /* right arrow */
		b_____,
		b__X__,
		b___X_,
		bXXXXX,
		b___X_,
		b__X__,
		b_____,
	},

	[0x7f] = { /* left arrow */
		b_____,
		b__X__,
		b_X___,
		bXXXXX,
		b_X___,
		b__X__,
		b_____,
	},
};
