/*
 * Copyright (c) 2021-2022 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/*
 * Unfortunately, there doesn't seem to be an exact definition of ITU zones.
 * In the USA, the ARRL's definition is what everyone uses as the official
 * one.
 */

#include "zones_impl.h"

static const struct zone canada[13] = {
	{ "NS", ZP_CONST, .constant = 9 },
	{ "QC", ZP_UNKNOWN }, /* 4 or 9 */
	{ "ON", ZP_UNKNOWN }, /* 3 or 4 */
	{ "MB", ZP_CONST, .constant = 3 },
	{ "SK", ZP_CONST, .constant = 3 },
	{ "AB", ZP_CONST, .constant = 2 },
	{ "BC", ZP_CONST, .constant = 2 },
	{ "NT", ZP_UNKNOWN }, /* 2 or 3 */
	{ "NB", ZP_CONST, .constant = 9 },
	{ "NL", ZP_CONST, .constant = 9 },
	{ "YT", ZP_CONST, .constant = 2 },
	{ "PE", ZP_CONST, .constant = 9 },
	{ "NU", ZP_UNKNOWN }, /* 3 or 4 */
};

static const struct zone usa[] = {
	/* call area 1 */
	{ "CT", ZP_CONST, .constant = 8 },
	{ "ME", ZP_CONST, .constant = 8 },
	{ "MA", ZP_CONST, .constant = 8 },
	{ "NH", ZP_CONST, .constant = 8 },
	{ "RI", ZP_CONST, .constant = 8 },
	{ "VT", ZP_CONST, .constant = 8 },

	/* call area 2 */
	{ "NJ", ZP_CONST, .constant = 8 },
	{ "NY", ZP_CONST, .constant = 8 },

	/* call area 3 */
	{ "DC", ZP_CONST, .constant = 8 }, /* not really a state */
	{ "DE", ZP_CONST, .constant = 8 },
	{ "MD", ZP_CONST, .constant = 8 },
	{ "PA", ZP_CONST, .constant = 8 },

	/* call area 4 */
	{ "AL", ZP_CONST, .constant = 8 },
	{ "FL", ZP_CONST, .constant = 8 },
	{ "GA", ZP_CONST, .constant = 8 },
	{ "KY", ZP_CONST, .constant = 8 },
	{ "NC", ZP_CONST, .constant = 8 },
	{ "SC", ZP_CONST, .constant = 8 },
	{ "TN", ZP_UNKNOWN }, /* 7 or 8 (lon >= -90) */
	{ "VA", ZP_CONST, .constant = 8 },

	/* call area 5 */
	{ "AR", ZP_UNKNOWN }, /* 7 or 8 (lon > -90) */
	{ "LA", ZP_UNKNOWN }, /* 7 or 8 (lon > -90) */
	{ "MS", ZP_UNKNOWN }, /* 7 or 8 (lon > -90) */
	{ "NM", ZP_CONST, .constant = 7 },
	{ "OK", ZP_CONST, .constant = 7 },
	{ "TX", ZP_CONST, .constant = 7 },

	/* call area 6 */
	{ "CA", ZP_CONST, .constant = 6 },

	/* call area 7 */
	{ "AZ", ZP_UNKNOWN }, /* 6 or 7 (lon > -110) */
	{ "ID", ZP_CONST, .constant = 6 },
	{ "MT", ZP_UNKNOWN }, /* 6 or 7 (lon > -110) */
	{ "NV", ZP_CONST, .constant = 6 },
	{ "OR", ZP_CONST, .constant = 6 },
	{ "UT", ZP_UNKNOWN }, /* 6 or 7 (lon > -110) */
	{ "WA", ZP_CONST, .constant = 6 },
	{ "WY", ZP_CONST, .constant = 7 },

	/* call area 8 */
	{ "MI", ZP_UNKNOWN }, /* 7 or 8 (lon >= -90) */
	{ "OH", ZP_CONST, .constant = 8 },
	{ "WV", ZP_CONST, .constant = 8 },

	/* call area 9 */
	{ "IL", ZP_UNKNOWN }, /* 7 or 8 (lon >= -90) */
	{ "IN", ZP_CONST, .constant = 8 },
	{ "WI", ZP_UNKNOWN }, /* 7 or 8 (lon >= -90) */

	/* call area 0 */
	{ "CO", ZP_CONST, .constant = 7 },
	{ "IA", ZP_CONST, .constant = 7 },
	{ "KS", ZP_CONST, .constant = 7 },
	{ "MN", ZP_UNKNOWN }, /* 7 or 8 (lon >= -90) */
	{ "MO", ZP_UNKNOWN }, /* 7 or 8 (lon >= -90) */
	{ "NE", ZP_CONST, .constant = 7 },
	{ "ND", ZP_CONST, .constant = 7 },
	{ "SD", ZP_CONST, .constant = 7 },
};

int ituz_by_state(int dxcc, const char *state)
{
	switch (dxcc) {
		case 1: /* Canada */
			return find_zone(state, canada, ARRAY_LEN(canada));
		case 6: /* Alaska */
			return -1; /* 1 or 2 */
		case 110: /* Hawaii */
			return 61;
		case 291: /* United States */
			return find_zone(state, usa, ARRAY_LEN(usa));
	}

	return -1;
}
