Lua hlog module
===============

Constest scripts, scripts run by run-script, and user configs all have
access to the 'hlog' module.  This file documents all the functions provided
by this module.


hlog.adif
---------

### `lookup_adif_mode(mode, submode)`
Looks up ADIF mode-submode pair and returns a hlog mode string.  E.g., ADIF
"SSB" & "LSB" will yield "LSB".

### `write_header()`
Returns string with an ADIF header.  This includes ADIF spec version,
program version, etc.

### `write_qso(qso, strict)`
Returns the qso structure serialized as ADIF.  The strict bool argument
controls whether the result generates the mode-submode field pair as
required by ADIF (strict=true), or if it simply puts the QSO mode into the
ADIF mode field (strict=false).


hlog.country
------------

### `lookup(call, state)`
Look up the "country data" for a specific callsign.  This matches the
callsign prefix with information such as dxcc, continent, itu, and cq zones.
Some dxcc entities span multiple itu or cq zones, and for those the state
argument can be used to further refine the lookup.

Returns a table with the following keys:

 * prefix: matched callsign prefix
 * name: normalized prefix for the matched entity (useful for entities
   with multiple callsign prefixes)
 * country: name of the entity associated with the prefix
 * continent
 * lat: degrees, N is positive
 * lon: degrees, E is positive
 * tz: seconds, E is positive
 * dxcc
 * cqz: may be missing if not known
 * itu: may be missing if not known


hlog.index
----------

### `history([call,] asc)`
Start iterating through QSO history.  If a callsign string argument is
supplied, iteration will be limited to only entries with that callsign.  The
`asc` argument controls whether iteration occurs oldest to newest (asc=true)
or newest to oldest (asc=false).

Returns the standard function & state interator values.


hlog.nvl
--------

### `clear(nvlist)`
Clears the passed in nvlist.


hlog.qso
--------

### `new()`
Returns a newly allocated qso structure.


hlog.sig
--------

### `lookup_pota(ref)`
Look up the park by its reference.  If found, returns a table with the
following keys, nil otherwise.

 * ref: the reference (e.g., K-8399)
 * name: name of park
 * locs: comma separated list of states (e.g., US-MA,US-NH)
 * active: bool indicating whether park can be activated
 * dxcc

### `lookup_sota(ref)`
Look up the summit by its reference.  If found, returns a table with the
following keys, nil otherwise.

 * ref: the reference (e.g., W1/CR-001)
 * name: name of summit
 * points


hlog.utils
----------

### `count_items(table)`
Returns the number of items in the passed in table.  Returns 0 if passed in
value is nil.

### `sorted_keys(table)`
Collects all the passed in table's keys, sorts them, and returns them as a
table.

### `split_on_commas(string)`
Splits the passed in strings on commas (,) and returns the result as a
table.  If the input doesn't contain any commas, the resulting table will
have only one entry.  If the input is nil, the result is nil.
