/*
 * Copyright (c) 2020-2021 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include <ncurses.h>

#include "status.h"

static WINDOW *win;
static size_t status_max_len;
static int status_color;

int status_init(int height, int width, int row, int col, int color)
{
	win = newwin(height, width, row, col);
	status_color = color;
	status_max_len = width;

	return 0;
}

void status_deinit(void)
{
	delwin(win);
}

void status_refresh(void)
{
	wrefresh(win);
}

static void do_display(const char *fmt, va_list ap, int color)
{
	char buf[status_max_len + 1];

	vsnprintf(buf, sizeof(buf), fmt, ap);

	wattrset(win, A_NORMAL | color);
	mvwprintw(win, 0, 0, "%-*s", status_max_len, buf);
	wattrset(win, A_NORMAL);
}

void status_display_error(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	do_display(fmt, ap, status_color);
	va_end(ap);
}

void status_display_info(const char *fmt, ...)
{
	va_list ap;

	va_start(ap, fmt);
	do_display(fmt, ap, 0);
	va_end(ap);
}
