/*
 * Copyright (c) 2020-2022 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef __XLUA_IMPL_H
#define __XLUA_IMPL_H

#include <jeffpc/types.h>

#include <hlog/qso.h>
#include <hlog-lua/xlua.h>

extern int xlua_load_script_internal(lua_State *L, const char *scriptname,
				     char *error, size_t errlen);

/* push various lib tables */
extern void xlua_pushlib_adif(lua_State *L);
extern void xlua_pushlib_country(lua_State *L);
extern void xlua_pushlib_index(lua_State *L);
extern void xlua_pushlib_nvl(lua_State *L);
extern void xlua_pushlib_qso(lua_State *L);
extern void xlua_pushlib_sig(lua_State *L);
extern void xlua_pushlib_utils(lua_State *L);

/* get various types */
extern struct qso *xlua_toqso(lua_State *L, int idx);

/* variants of public non-throwing push functions */
extern void xlua_pushnvl_throws(lua_State *L, struct nvlist *nvl, bool ro);
extern void xlua_pushqso_throws(lua_State *L, struct qso *qso, bool ro);
extern void xlua_pushtime_throws(lua_State *L, struct time *time, bool ro);

/*
 * check that the stack is in a reasonable state [-0,+0,e]
 *
 * - make sure that there is the correct number of arguments
 * - make sure that there is enough room for any pushes
 */
static inline void xlua_checkstack(lua_State *L, int minnargs, int maxnargs,
				   int npushes)
{
	int nargs = lua_gettop(L);

	if ((nargs < minnargs) || (nargs > maxnargs))
		(void) luaL_error(L, "bad number of arguments "
				  "(expected %d..%d, got %d)", minnargs,
				  maxnargs, nargs);

	ASSERT3S(lua_checkstack(L, npushes), !=, 0);
}

#endif
